/****************************************************************************
**
** $Id: noncvslistview.cpp,v 1.11 2001/06/28 23:03:33 joseh Exp $
**
** Copyright (C) 2001 The LinCVS development team.
**    Tilo Riemer <riemer@lincvs.org>
**    Falk Brettschneider <gigafalk@yahoo.com>
**    Wim Delvaux <wim.delvaux@chello.be>
**    Jose Hernandez <joseh@tesco.net>
**    Helmut Koll <HelmutKoll@web.de>
**    Sven Trogisch <trogisch@iapp.de>
**
**
**----------------------------------------------------------------------------
**
**----------------------------------------------------------------------------
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as
** published by the Free Software Foundation; either version 2 of the
** License, or (at your option) any later version.
**
*****************************************************************************/


#include "config.h"
#include "ac_system_defs.h"

#include <stdio.h>
#include <qlistview.h>
#include <qevent.h>
#include <qpopupmenu.h>

#include "noncvslistview.h"
#include "globals.h"
#include "PixmapCache.h"


QPopupMenu * NonCvsListView::PopupMenu = 0;

NonCvsListView::NonCvsListView( QWidget * Parent ) : 
	QListView( Parent )
{
	/* Use extended selection mode.  When we click on this
	 * window we usually want to operate on a single file by using
	 * the menu unless the user uses the shift or control key.
	 * We don't want to use the standard multiple selection mode
	 * because the point in having a tree view is so we can perform
	 * the operation on the whole directory hierarchy in one go.
	 */
	setSelectionMode (QListView::Extended);

	if( PopupMenu == 0 ){
	   PopupMenu = new QPopupMenu(0);
	   PopupMenu->insertItem(findEmbeddedPixmap ("FileASCII16x16"),
				 tr("Add ASCII file"), this,
				 SLOT(internalAddFiles()));
	   PopupMenu->insertItem(findEmbeddedPixmap ("FileBinary16x16"),
				 tr("Add binary file"), this,
				 SLOT(internalAddBinFiles()));
	   PopupMenu->insertSeparator();
	   PopupMenu->insertItem(findEmbeddedPixmap ("Delete16x16"),
				 tr("Delete from disk"), this,
				 SLOT(internalDeleteNonCvsFiles()));
	}

        prevCount = 0;
        addColumn(tr("Name"));
        addColumn(tr("Modified (local)"));
        addColumn(tr("Modified (UTC)"));
        setColumnWidth(0,NONCVSLISTVIEW_TABWIDTH_NAME);
        setColumnWidth(1,NONCVSLISTVIEW_TABWIDTH_MODLOCAL);
        setColumnWidth(2,NONCVSLISTVIEW_TABWIDTH_MODUTC);
        setAllColumnsShowFocus( TRUE );

        connect( this, SIGNAL( doubleClicked( QListViewItem *) ),
                        this, SLOT( selFile( QListViewItem * ) ) );
}

NonCvsListView::~NonCvsListView() {

}

void NonCvsListView::viewportMousePressEvent( QMouseEvent * ev) {

	if( ev->button() == QMouseEvent::RightButton ) {
	  // popup menu
	  QPoint offset(10, 5);
	  PopupMenu->popup(mapToGlobal(ev->pos()) + offset);
	  PopupMenu->show();
	} else {
	  QListView::viewportMousePressEvent( ev );
	}
}

void NonCvsListView::internalAddFiles( void ) {
	emit addFiles();
}

void NonCvsListView::internalAddBinFiles( void ) {
	emit addBinFiles();
}


void NonCvsListView::internalDeleteNonCvsFiles (void)
{
	emit deleteNonCvsFiles();
}


void NonCvsListView::selFile( QListViewItem * it ) {
        clearSelection();
	emit doubleClickedFile( it->text(0) );
}

void NonCvsListView::signalState( void ) {
        if( childCount() != prevCount ) {
          prevCount = childCount();
          emit itemCount( prevCount );
        }
}
