/****************************************************************************
**
** $Id: settings.cpp,v 1.11 2001/07/12 00:31:06 joseh Exp $
**
** Copyright (C) 2001 The LinCVS development team.
**    Tilo Riemer <riemer@lincvs.org>
**    Falk Brettschneider <gigafalk@yahoo.com>
**    Wim Delvaux <wim.delvaux@chello.be>
**    Jose Hernandez <joseh@tesco.net>
**    Helmut Koll <HelmutKoll@web.de>
**    Sven Trogisch <trogisch@iapp.de>
**
**
**----------------------------------------------------------------------------
**
**----------------------------------------------------------------------------
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as
** published by the Free Software Foundation; either version 2 of the
** License, or (at your option) any later version.
**
*****************************************************************************/

#include "config.h"
#include "ac_system_defs.h"

#include <stdio.h>


/* Version Control ID */
#ifndef lint
static char vcid[] = "$Id: settings.cpp,v 1.11 2001/07/12 00:31:06 joseh Exp $";
#endif /* lint */

/*---------------------------------------------------------------------------*
 *  Include Files                                                            *
 *---------------------------------------------------------------------------*/

#include <qbuttongroup.h>
#include <qpushbutton.h>
#include <qhbox.h>
#include <qvbox.h>
#include <qlabel.h>
#include <qlayout.h> 
#include <qwidget.h>
#include <qstringlist.h> 
#include <qstring.h>
#include <qmessagebox.h>
#include <qlist.h>
#include <qtooltip.h>

#include "settings.h"
#include "lincvs.h"
#include "cfg.h"

/*---------------------------------------------------------------------------*
 *  Definitions, Macros                                                      *
 *---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*
 *  typedefs                                                                 *
 *---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*
 *  global Variables                                                         *
 *---------------------------------------------------------------------------*/
extern Config *conf;

const QString   pwd   = "Password";
const QString   rmsh  = "Remote Shell";
const QString   loc   = "Local/NFS";

/*---------------------------------------------------------------------------*
 *  Prototypes / classes                                                     *
 *---------------------------------------------------------------------------*/




/*---------------------------------------------------------------------------*/
/*!
\fn			cvsEnvSet::cvsEnvSet(QWidget *parent, const char* name)
\brief		ctor. Layout for timer and profile setting.

\param		*parent		
\param		name		

\return		
<BR><HR>*/
/*---------------------------------------------------------------------------*/

cvsEnvSet::cvsEnvSet(QStringList *userList,
                     QStringList *serverList,
                     QStringList *repositoryList,
                     QStringList *profileList,
                     QWidget *parent, const char* name)
    :QDialog( parent , name, TRUE, 0 )
{ 
  p_userList       = userList;
  p_serverList     = serverList;
  p_repositoryList = repositoryList;
  p_profileList   = profileList;
  
  setCaption (tr("LinCVS Settings"));
  
// ----- init qlist --------------------------------------------------------

  if (!conf->list.isEmpty() )
    tmpList = conf->list; 
  
  delList.clear();
  
// ----- layout ------------------------------------------------------------

  QBoxLayout * vboxLyOut = new QVBoxLayout( this );
  vboxLyOut->setAutoAdd( TRUE );

// ----- timer ------------------------------------------------------------

  QVBox *vboxTimer = new QVBox (this);

  QGroupBox *grp0 = new QButtonGroup( 1, QGroupBox::Horizontal, vboxTimer );
  grp0->setTitle (tr("timer"));
  
  QLabel *lb = new QLabel ("select a time for automatic update of workbench",
                           grp0);
  lb->setAlignment ( QLabel::AlignHCenter );
 
  QHBox *hboxTimer = new QHBox (grp0);
  dummy = new QWidget(hboxTimer);
  dummy = new QWidget(hboxTimer);
  timSec = new QLCDNumber ( 4, hboxTimer );
  timSec->setSegmentStyle ( QLCDNumber::Flat );

  lb = new QLabel (tr(" [sec]"), hboxTimer);
  dummy = new QWidget(hboxTimer);
  
  updTimer = new QSlider (3, 60, 1, CHECKSTATUSINTERVALL / 1000,
                           QSlider::Horizontal, grp0, "myslider");
  connect( updTimer, SIGNAL(valueChanged(int)), SLOT(dispVal()) ); 
  dispVal();  // to init display of lcd 
  
  dummy = new QWidget(this);
  dummy->setFixedHeight(grp0->height()/2);
 
// ----- profile -----------------------------------------------------------

  QVBox *vboxRoot = new QVBox (this);

  QGroupBox *grp1 = new QButtonGroup( 1, QGroupBox::Horizontal, vboxRoot );
  grp1->setTitle (tr("profile"));

  QHBox *hgrp1 = new QHBox (grp1);
  lb = new QLabel ("Profile Name ", hgrp1);
  lb->setMinimumWidth(width()/3.);
  selName   = new QComboBox ( TRUE, hgrp1);
  selName->setMinimumWidth(width()*2.0/3.0);
  selName->setAutoCompletion ( TRUE );
  connect(selName, SIGNAL (activated ( const QString & ) ), 
          this, SLOT (actNameItem(const QString &  )) );

  insertHLine(grp1);
  QHBox *hgrp2 = new QHBox (grp1);
  lb = new QLabel ("User ", hgrp2);
  lb->setMinimumWidth(width()/3.);
  selUser   = new QComboBox ( TRUE, hgrp2);
  selUser->setMinimumWidth(width()*2.0/3.0);
  selUser->setAutoCompletion ( TRUE );

  QHBox *hgrp3 = new QHBox (grp1);
  lb = new QLabel ("Server ", hgrp3);
  lb->setMinimumWidth(width()/3.);
  selServer = new QComboBox ( TRUE, hgrp3);
  selServer->setMinimumWidth(width()*2.0/3.0);
  selServer->setAutoCompletion ( TRUE );

  QHBox *hgrp4 = new QHBox (grp1);
  lb = new QLabel ("Repository ", hgrp4);
  lb->setMinimumWidth(width()/3.);
  selRepository = new QComboBox ( TRUE, hgrp4);
  selRepository->setMinimumWidth(width()*2.0/3.0);
  selRepository->setAutoCompletion ( TRUE );

  connect(selRepository, SIGNAL (activated ( const QString & ) ), 
          this, SLOT (actItems(const QString &  )) );


  QHBox *hgrp5 = new QHBox (grp1);
  lb = new QLabel ("Method ", hgrp5);
  lb->setMinimumWidth(width()/3.);
  selMethod = new QComboBox ( TRUE, hgrp5);
  selMethod->setMinimumWidth(width()*2.0/3.0);
  selMethod->setAutoCompletion ( TRUE );
  selMethod->setInsertionPolicy ( QComboBox::NoInsertion );

/*! init internal methods List */
  methodList = new QStringList();
  methodList->append(pwd);         // MODE_PSERVER
  methodList->append(rmsh);        // MODE_RSH
  methodList->append(loc);         // MODE_LOCAL
  selMethod->insertStringList(*methodList);

  updateComboList();

  insertHLine(grp1);

/*! add pushbuttons 'modify', 'add' and 'delete` */
  QHBox *hgrpBt = new QHBox (grp1);

  dummy = new QWidget(hgrpBt);
  QPushButton *pb_mod = new QPushButton("Modify", hgrpBt);
  connect(pb_mod, SIGNAL(clicked()), this, SLOT(modList()));
  dummy = new QWidget(hgrpBt);

  QPushButton *pb_add = new QPushButton("Add", hgrpBt);
  connect(pb_add, SIGNAL(clicked()), this, SLOT(addItem()));

  dummy = new QWidget(hgrpBt);
  QPushButton *pb_delete = new QPushButton("Delete", hgrpBt);
  connect(pb_delete, SIGNAL(clicked()), this, SLOT(deleteItem()));
  dummy = new QWidget(hgrpBt);
   

// ------------------------------------------------------------------------

/*! insert a space between the button groups */
  dummy = new QWidget(this);
  dummy->setFixedHeight(grp0->height()/2);
  
/*! add pushbuttons 'ok' and 'cancel` */
  QHBox *hboxButton = new QHBox (this);

  QGroupBox *grp = new QButtonGroup( 1, QGroupBox::Vertical, hboxButton );

  dummy = new QWidget(grp);
  dummy = new QWidget(grp);
  QPushButton *pb_ok = new QPushButton("OK", grp);
  connect(pb_ok, SIGNAL(clicked()), this, SLOT(ok()));

  dummy = new QWidget(grp);
  QPushButton *pb_cancel = new QPushButton("Cancel", grp);
  connect(pb_cancel, SIGNAL(clicked()), this, SLOT(cancel()));
  dummy = new QWidget(grp);
  dummy = new QWidget(grp);
 
  this->resize(300, 150);


  QToolTip::add(timSec, tr("Shows the update time set with the slider") );
  QToolTip::add(updTimer, tr("select an update time for background update of workbench") );
  QToolTip::add(selName, tr("select or enter a name for the following entries of : user - server - repository - method") );
  QToolTip::add(selUser, tr("select or enter a user name") );
  QToolTip::add(selServer, tr("select or enter a server name") );
  QToolTip::add(selRepository, tr("select or enter a CVS repository") );
  QToolTip::add(selMethod, tr("select a connect method") );
  QToolTip::add(pb_mod, tr("modify the seleted entry then push here") );
  QToolTip::add(pb_add, tr("add new entries in the fields above then push") );
  QToolTip::add(pb_delete, tr("push if you want to delete the shown entry") );
  QToolTip::add(pb_ok, tr("push if you want to commit all manipulations") );
  QToolTip::add(pb_cancel, tr("push if you want to cancel all manipulations") );
}



/*---------------------------------------------------------------------------*/
/*!
\fn			void cvsEnvSet::ok(void)
\brief		Accepts the inputs and leaves.

<BR><HR>*/
/*---------------------------------------------------------------------------*/

void cvsEnvSet::ok(void)
{
  lincvsProfile *profile;
  lincvsProfile *dprofile;

/*! timer */
  int sec = updTimer->value();
  CHECKSTATUSINTERVALL = sec * 1000;
  

  conf->list.clear();
  if (!tmpList.isEmpty())
  {
    conf->list = tmpList;
  }
  
// ----- delete items in stringlists ------------------------------------

  for ( dprofile=delList.first(); dprofile != 0; dprofile=delList.next() )
  {
    deleteEntry(p_profileList,    dprofile->name());
    deleteEntry(p_userList,       dprofile->user());
    deleteEntry(p_serverList,     dprofile->server());
    deleteEntry(p_repositoryList, dprofile->repository());
  }

// ----- insert items in stringlists ------------------------------------

  for ( profile=conf->list.first(); profile != 0; profile=conf->list.next() )
  {
    conf->insertEntry(p_profileList,    profile->name().stripWhiteSpace() );
    conf->insertEntry(p_userList,       profile->user().stripWhiteSpace() );
    conf->insertEntry(p_serverList,     profile->server().stripWhiteSpace() );
    conf->insertEntry(p_repositoryList, profile->repository().stripWhiteSpace() );
  }

  close();
}



/*---------------------------------------------------------------------------*/
/*!
\fn			void cvsEnvSet::cancel(void)
\brief		Discards any input and leaves.

<BR><HR>*/
/*---------------------------------------------------------------------------*/

void cvsEnvSet::cancel(void)
{
  close();
}



/*---------------------------------------------------------------------------*/
/*!
\fn			void cvsEnvSet::insertHLine()
\brief		Inserts a horizontal line.

<BR><HR>*/
/*---------------------------------------------------------------------------*/

void cvsEnvSet::insertHLine(QWidget *lparent)
{
  dummy = new QWidget(lparent);

  QFrame *emptyFrame = new QFrame( lparent );
  emptyFrame->setFrameStyle( QFrame::Sunken | QFrame::HLine);

  dummy = new QWidget(lparent);
}



/*---------------------------------------------------------------------------*/
/*!
\fn			void cvsEnvSet::dispVal()
\brief		Shows the value of QSlider in LCD number.

<BR><HR>*/
/*---------------------------------------------------------------------------*/

void cvsEnvSet::dispVal(void)
{
  int sec = updTimer->value();
  timSec->display(sec);
}



/*---------------------------------------------------------------------------*/
/*!
\fn			int cvsEnvSet::getMethod(QString text)
\brief		returns a number for the method string

\param		text		method string

\return		
<BR><HR>*/
/*---------------------------------------------------------------------------*/

int cvsEnvSet::getMethod(QString text)
{
  accessMethod  accmethod = Unknown;
  
  if (text == pwd)  accmethod = Password;
  if (text == rmsh) accmethod = RemoteShell;
  if (text == loc)  accmethod = Local_NFS;

  if (accmethod == Unknown)
  {
    QMessageBox::warning(0, tr("LinCVS Settings"), tr("No such access method.\n") 
                                 + "Please select one from list", 0);
  }
  return accmethod;
}



/*---------------------------------------------------------------------------*/
/*!
\fn			void cvsEnvSet::modList(void)
\brief		modify an entry in the list


<BR><HR>*/
/*---------------------------------------------------------------------------*/

void cvsEnvSet::modList(void)
{
  lincvsProfile *mdprofile;

  if (tmpList.isEmpty()) return;

/*! get current position */
  uint idx = tmpList.at();
  if (idx >= tmpList.count()) mdprofile = tmpList.at(0);
  else mdprofile = tmpList.at(idx);
  if (getMethod(selMethod->currentText()) == Unknown) return;
printf("2\n");

/*! old value to delete */
  delList.append(new lincvsProfile(mdprofile->name(),
                                   mdprofile->method(),  
                                   mdprofile->user(),
                                   mdprofile->server(), 
                                   mdprofile->repository()) );
printf("3\n");
  tmpList.remove (idx);
printf("4\n");

/*! new value to insert */
  tmpList.insert(idx, new lincvsProfile(selName->currentText(),
                                        getMethod(selMethod->currentText()),  
                                        selUser->currentText(),
                                        selServer->currentText(), 
                                        selRepository->currentText()) );
printf("5\n");
  updateComboList();
}



/*---------------------------------------------------------------------------*/
/*!
\fn			void cvsEnvSet::addItem(void)
\brief		Adds a new item to the repository list. The item must be different
          from the existing ones.

<BR><HR>*/
/*---------------------------------------------------------------------------*/

void cvsEnvSet::addItem(void)
{
  lincvsProfile *profile;

  QString tmp;
  pIndex = 0;

// test if entry is acceptable
  QString item = selName->currentText();
  if (item.isEmpty())
  {
    QMessageBox::warning(0, tr("LinCVS Settings"), tr("No profile name specified.\n"), 0);
    return;
  }
  
  if (getMethod(selMethod->currentText()) == Unknown) return;
  
// test if name already exists!!
/*! search index in qlist */
  for ( profile=tmpList.first(); profile != 0; profile=tmpList.next() )
  {
    if (profile->name() == item )
    {
      pIndex = tmpList.current();
      break;
    }
  }
    
  if (pIndex == 0)
  {
    tmpList.append(new lincvsProfile(selName->currentText(),
                                     getMethod(selMethod->currentText()),  
                                     selUser->currentText(),
                                     selServer->currentText(), 
                                     selRepository->currentText()) );
  }else
  {
    QMessageBox::warning(0, tr("LinCVS Settings"), 
                         tr("Profile already exists with this name.\n") 
                         + "Please select a new name", 0);
  }
  
  updateComboList();
}



/*---------------------------------------------------------------------------*/
/*!
\fn			void cvsEnvSet::actItems(const QString &item)
\brief		Slot when enter pressed.

\param		&item		

<BR><HR>*/
/*---------------------------------------------------------------------------*/

void cvsEnvSet::actItems(const QString&)
{
  addItem();
}



/*---------------------------------------------------------------------------*/
/*!
\fn			void cvsEnvSet::deleteItem(void)
\brief		Deletes the current item from the combo box in group CVSROOT form 
          the repository list.

<BR><HR>*/
/*---------------------------------------------------------------------------*/

void cvsEnvSet::deleteItem(void)
{
  if (tmpList.isEmpty()) return;

/*! store deleted items temp. in list */
  delList.append(new lincvsProfile(selName->currentText(),
                                   getMethod(selMethod->currentText()),  
                                   selUser->currentText(),
                                   selServer->currentText(), 
                                   selRepository->currentText()) );

  tmpList.remove(pIndex);
  updateComboList();
}



/*---------------------------------------------------------------------------*/
/*!
\fn			void cvsEnvSet::actNameItem(const QString &item)
\brief		Searches the index of the actual item of profile name in the qlist
          lincvsProfile and sets the other combo boxes to the appropriate values
          resulting from qlist.

\param		&item		

<BR><HR>*/
/*---------------------------------------------------------------------------*/

void cvsEnvSet::actNameItem(const QString &item)
{
  lincvsProfile *profile;

  QString tmp;
  pIndex = tmpList.first();

/*! search index in qlist */
  for ( profile=tmpList.first(); profile != 0; profile=tmpList.next() )
  {
    if (profile->name() == item )
    {
      pIndex = tmpList.current();
      break;
    }
  }
    

/*! search item in combobox name */
  int i;
  for (i=0; i<selName->count(); i++)
  {
    tmp=selName->text(i);
    if ( tmp==pIndex->name() )
    {
      selName->setCurrentItem(i);
      break;
    }
  }
  

/*! search item in combobox user */
  for (i=0; i<selUser->count(); i++)
  {
    tmp=selUser->text(i);
    if ( tmp==pIndex->user() )
    {
      selUser->setCurrentItem(i);
      break;
    }
  }

/*! search item in combobox server */
  for (i=0; i<selServer->count(); i++)
  {
    tmp=selServer->text(i);
    if ( tmp==pIndex->server() )
    {
      selServer->setCurrentItem(i);
      break;
    }
  }

/*! search item in combobox repository */
  for (i=0; i<selRepository->count(); i++)
  {
    tmp=selRepository->text(i);
    if ( tmp==pIndex->repository() )
    {
      selRepository->setCurrentItem(i);
      break;
    }
  }

/*! set item in combobox method */
  selMethod->setCurrentItem( pIndex->method() );
}



/*---------------------------------------------------------------------------*/
/*!
\fn			void cvsEnvSet::updateComboList(void)
\brief		Updates the combolists with the modified contents of tmpList.


<BR><HR>*/
/*---------------------------------------------------------------------------*/

void cvsEnvSet::updateComboList(void)
{
  QString currProfile = selName->currentText();

  lincvsProfile *profile;

  selName->clear();
  selUser->clear();
  selServer->clear();
  selRepository->clear();
  
  if (tmpList.isEmpty()) 
    return;
  

  for ( profile=tmpList.first(); profile != 0; profile=tmpList.next() )
  {
    selName->insertItem(profile->name());
    selUser->insertItem(profile->user());
    selServer->insertItem(profile->server());
    selRepository->insertItem(profile->repository());
  }
  actNameItem(currProfile);
}



/*---------------------------------------------------------------------------*/
/*!
\fn			void cvsEnvSet::deleteEntry(QStringList &list, const QString item)
\brief		searches for the marked entries to delete and if found does it.

\param		&list		list from where the items are to be deleted
\param		item		item to delete

<BR><HR>*/
/*---------------------------------------------------------------------------*/

void cvsEnvSet::deleteEntry(QStringList* list, const QString item)
{
  bool found;
  found = FALSE;
  
 /*! search the entire list for the item */
  for ( QStringList::Iterator it =  list->begin(); it != list->end(); ++it )
  {
    if (item.compare(*it) == 0) 
    {
      found = TRUE;
      break;
    }
  }
  
  if (found)
  {
    list->remove(item);
  }
}



/*--------- END OF FILE -----------------------------------------------------*/
